@@ -1735,8 +1735,11 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			return_val = FALSE;
 			gtk_signal_emit (GTK_OBJECT (eti), eti_signals [CLICK],
 					 row, view_to_model_col(eti, col), &button, &return_val);
-			if (return_val)
+
+			if (return_val) {
+				eti->click_count = 0;
 				return TRUE;
+			}
 
 			gtk_object_get(GTK_OBJECT(eti->selection),
 				       "cursor_row", &cursor_row,
@@ -1954,6 +1957,7 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 
 		switch (e->key.keyval){
 		case GDK_Left:
+		case GDK_KP_Left:
 			if (eti_editing (eti)) {
 				handled = FALSE;
 				break;
@@ -1964,8 +1968,9 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			if ((!return_val) && eti->cursor_mode != E_CURSOR_LINE && cursor_col != view_to_model_col(eti, 0))
 				eti_cursor_move_left (eti);
 			break;
-			
+
 		case GDK_Right:
+		case GDK_KP_Right:
 			if (eti_editing (eti)) {
 				handled = FALSE;
 				break;
@@ -1978,7 +1983,9 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			break;
 			
 		case GDK_Up:
+		case GDK_KP_Up:
 		case GDK_Down:
+		case GDK_KP_Down:
 			return_val = e_selection_model_key_press(E_SELECTION_MODEL (eti->selection), (GdkEventKey *) e);
 			break;
 		case GDK_Home:
