@@ -528,57 +528,47 @@ item_key_press (ETableItem *eti, int row, int col, GdkEvent *event, ETree *et)
 {
 	int return_val = 0;
 	GdkEventKey *key = (GdkEventKey *) event;
-	GdkEventButton click;
 	ETreePath path;
+	int y, row_local, col_local;
+	GtkAdjustment *vadj;
 
 	switch (key->keyval) {
 	case GDK_Page_Down:
-		gtk_adjustment_set_value(
-			gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas)),
-			CLAMP(gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->value +
-			      (gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->page_size - 20),
-			      0,
-			      gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->upper -
-			      gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->page_size));
-		click.type = GDK_BUTTON_PRESS;
-		click.window = GTK_LAYOUT (et->priv->table_canvas)->bin_window;
-		click.send_event = key->send_event;
-		click.time = key->time;
-		click.x = 30;
-		click.y = gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->page_size - 1;
-		click.state = key->state;
-		click.button = 1;
-		gtk_widget_event(GTK_WIDGET(et->priv->table_canvas),
-				 (GdkEvent *) &click);
+		vadj = gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas));
+		y = CLAMP(vadj->value + (2 * vadj->page_size - 50), 0, vadj->upper);
+		y -= vadj->value;
+		e_tree_get_cell_at (et, 30, y, &row_local, &col_local);
+		row_local = e_tree_view_to_model_row (et, row_local);
+		col_local = e_selection_model_cursor_col (E_SELECTION_MODEL (et->priv->selection));
+		e_selection_model_select_as_key_press (E_SELECTION_MODEL (et->priv->selection), row_local, col_local, key->state);
 		return_val = 1;
 		break;
 	case GDK_Page_Up:
-		gtk_adjustment_set_value(
-			gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas)),
-			gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->value -
-			(gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->page_size - 20));
-		click.type = GDK_BUTTON_PRESS;
-		click.window = GTK_LAYOUT (et->priv->table_canvas)->bin_window;
-		click.send_event = key->send_event;
-		click.time = key->time;
-		click.x = 30;
-		click.y = 1;
-		click.state = key->state;
-		click.button = 1;
-		gtk_widget_event(GTK_WIDGET(et->priv->table_canvas),
-				 (GdkEvent *) &click);
+		vadj = gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas));
+		y = CLAMP(vadj->value - (vadj->page_size - 50), 0, vadj->upper);
+		y -= vadj->value;
+		e_tree_get_cell_at (et, 30, y, &row_local, &col_local);
+		row_local = e_tree_view_to_model_row (et, row_local);
+		col_local = e_selection_model_cursor_col (E_SELECTION_MODEL (et->priv->selection));
+		e_selection_model_select_as_key_press (E_SELECTION_MODEL (et->priv->selection), row_local, col_local, key->state);
 		return_val = 1;
 		break;
 	case '=':
 	case GDK_Right:
-		path = e_tree_table_adapter_node_at_row(et->priv->etta, row);
-		e_tree_table_adapter_node_set_expanded (et->priv->etta, path, TRUE);
+		if (row != -1) {
+			path = e_tree_table_adapter_node_at_row(et->priv->etta, row);
+			if (path)
+				e_tree_table_adapter_node_set_expanded (et->priv->etta, path, TRUE);
+		}
 		return_val = 1;
 		break;
 	case '-':
 	case GDK_Left:
-		path = e_tree_table_adapter_node_at_row(et->priv->etta, row);
-		e_tree_table_adapter_node_set_expanded (et->priv->etta, path, FALSE);
+		if (row != -1) {
+			path = e_tree_table_adapter_node_at_row(et->priv->etta, row);
+			if (path)
+				e_tree_table_adapter_node_set_expanded (et->priv->etta, path, FALSE);
+		}
 		return_val = 1;
 		break;
 	default:
@@ -1097,7 +1087,7 @@ e_tree_construct_from_spec_file (ETree *e_tree, ETreeModel *etm, ETableExtras *e
 GtkWidget *
 e_tree_new (ETreeModel *etm, ETableExtras *ete, const char *spec, const char *state)
 {
-	ETree *e_tree;
+	ETree *e_tree, *ret_val;
 
 	g_return_val_if_fail(etm != NULL, NULL);
 	g_return_val_if_fail(E_IS_TREE_MODEL(etm), NULL);
@@ -1106,15 +1096,19 @@ e_tree_new (ETreeModel *etm, ETableExtras *ete, const char *spec, const char *st
 
 	e_tree = gtk_type_new (e_tree_get_type ());
 
-	e_tree = e_tree_construct (e_tree, etm, ete, spec, state);
+	ret_val = e_tree_construct (e_tree, etm, ete, spec, state);
 
-	return GTK_WIDGET (e_tree);
+	if (ret_val == NULL) {
+		gtk_object_unref (GTK_OBJECT (e_tree));
+	}
+
+	return (GtkWidget *) ret_val;
 }
 
 GtkWidget *
 e_tree_new_from_spec_file (ETreeModel *etm, ETableExtras *ete, const char *spec_fn, const char *state_fn)
 {
-	ETree *e_tree;
+	ETree *e_tree, *ret_val;
 
 	g_return_val_if_fail(etm != NULL, NULL);
 	g_return_val_if_fail(E_IS_TREE_MODEL(etm), NULL);
@@ -1123,9 +1117,13 @@ e_tree_new_from_spec_file (ETreeModel *etm, ETableExtras *ete, const char *spec_
 
 	e_tree = gtk_type_new (e_tree_get_type ());
 
-	e_tree = e_tree_construct_from_spec_file (e_tree, etm, ete, spec_fn, state_fn);
+	ret_val = e_tree_construct_from_spec_file (e_tree, etm, ete, spec_fn, state_fn);
 
-	return GTK_WIDGET (e_tree);
+	if (ret_val == NULL) {
+		gtk_object_unref (GTK_OBJECT (e_tree));
+	}
+
+	return (GtkWidget *) ret_val;
 }
 
 void
@@ -1166,6 +1164,8 @@ e_tree_get_cursor (ETree *e_tree)
 	gtk_object_get(GTK_OBJECT(e_tree->priv->selection),
 		       "cursor_row", &row,
 		       NULL);
+	if (row == -1)
+		return NULL;
 	path = e_tree_table_adapter_node_at_row(E_TREE_TABLE_ADAPTER(e_tree->priv->etta), row);
 	path = e_tree_sorted_view_to_model_path(e_tree->priv->sorted, path);
 	return path;
@@ -1199,34 +1199,6 @@ e_tree_selected_path_foreach     (ETree *e_tree,
 }
 #endif
 
-gint
-e_tree_selected_count     (ETree *e_tree)
-{
-	g_return_val_if_fail(e_tree != NULL, -1);
-	g_return_val_if_fail(E_IS_TREE(e_tree), -1);
-
-	return e_selection_model_selected_count(E_SELECTION_MODEL (e_tree->priv->selection));
-}
-
-void
-e_tree_select_all (ETree *tree)
-{
-	g_return_if_fail (tree != NULL);
-	g_return_if_fail (E_IS_TREE (tree));
-
-	e_selection_model_select_all (E_SELECTION_MODEL (tree->priv->selection));
-}
-
-void
-e_tree_invert_selection (ETree *tree)
-{
-	g_return_if_fail (tree != NULL);
-	g_return_if_fail (E_IS_TREE (tree));
-
-	e_selection_model_invert_selection (E_SELECTION_MODEL (tree->priv->selection));
-}
-
-
 EPrintable *
 e_tree_get_printable (ETree *e_tree)
 {
@@ -1493,6 +1465,41 @@ e_tree_get_tooltip (ETree *et)
 	return E_CANVAS(et->priv->table_canvas)->tooltip_window;
 }
 
+/**
+ * e_tree_get_model:
+ * @et: the ETree
+ *
+ * Returns the model upon which this ETree is based.
+ *
+ * Returns: the model
+ **/
+ETreeModel *
+e_tree_get_model (ETree *et)
+{
+	g_return_val_if_fail (et != NULL, NULL);
+	g_return_val_if_fail (E_IS_TREE (et), NULL);
+
+	return et->priv->model;
+}
+
+/**
+ * e_tree_get_selection_model:
+ * @et: the ETree
+ *
+ * Returns the selection model of this ETree.
+ *
+ * Returns: the selection model
+ **/
+ESelectionModel *
+e_tree_get_selection_model (ETree *et)
+{
+	g_return_val_if_fail (et != NULL, NULL);
+	g_return_val_if_fail (E_IS_TREE (et), NULL);
+
+	return et->priv->selection;
+}
+
+
 struct _ETreeDragSourceSite
 {
 	GdkModifierType    start_button_mask;
@@ -1780,6 +1787,11 @@ e_tree_get_cell_at (ETree *tree,
 	/* FIXME it would be nice if it could handle a NULL row_return or
 	 * col_return gracefully.  */
 
+	if (row_return)
+		*row_return = -1;
+	if (col_return)
+		*col_return = -1;
+
 	x += GTK_LAYOUT(tree->priv->table_canvas)->hadjustment->value;
 	y += GTK_LAYOUT(tree->priv->table_canvas)->vadjustment->value;
 	e_table_item_compute_location(E_TABLE_ITEM(tree->priv->item), &x, &y, row_return, col_return);
@@ -1805,6 +1817,8 @@ e_tree_get_cell_geometry (ETree *tree,
 {
 	g_return_if_fail (tree != NULL);
 	g_return_if_fail (E_IS_TREE (tree));
+	g_return_if_fail (row >= 0);
+	g_return_if_fail (col >= 0);
 
 	/* FIXME it would be nice if it could handle a NULL row_return or
 	 * col_return gracefully.  */
@@ -1908,8 +1922,7 @@ et_drag_motion(GtkWidget *widget,
 			    y,
 			    &row,
 			    &col);
-	if (et->priv->drop_row >= 0 && et->priv->drop_col >= 0 &&
-	    row != et->priv->drop_row && col != et->priv->drop_row) {
+	if (row != et->priv->drop_row && col != et->priv->drop_row) {
 		gtk_signal_emit (GTK_OBJECT (et),
 				 et_signals [TREE_DRAG_LEAVE],
 				 et->priv->drop_row,
@@ -1925,17 +1938,16 @@ et_drag_motion(GtkWidget *widget,
 	et->priv->drop_row = row;
 	et->priv->drop_path = path;
 	et->priv->drop_col = col;
-	if (row >= 0 && col >= 0)
-		gtk_signal_emit (GTK_OBJECT (et),
-				 et_signals [TREE_DRAG_MOTION],
-				 et->priv->drop_row,
-				 et->priv->drop_path,
-				 et->priv->drop_col,
-				 context,
-				 x,
-				 y,
-				 time,
-				 &ret_val);
+	gtk_signal_emit (GTK_OBJECT (et),
+			 et_signals [TREE_DRAG_MOTION],
+			 et->priv->drop_row,
+			 et->priv->drop_path,
+			 et->priv->drop_col,
+			 context,
+			 x,
+			 y,
+			 time,
+			 &ret_val);
 	return ret_val;
 }
 
@@ -1958,8 +1970,7 @@ et_drag_drop(GtkWidget *widget,
 	path = e_tree_table_adapter_node_at_row(et->priv->etta, row);
 	path = e_tree_sorted_view_to_model_path(et->priv->sorted, path);
 
-	if (et->priv->drop_row >= 0 && et->priv->drop_col >= 0 &&
-	    row != et->priv->drop_row && col != et->priv->drop_row) {
+	if (row != et->priv->drop_row && col != et->priv->drop_row) {
 		gtk_signal_emit (GTK_OBJECT (et),
 				 et_signals [TREE_DRAG_LEAVE],
 				 et->priv->drop_row,
@@ -1967,32 +1978,30 @@ et_drag_drop(GtkWidget *widget,
 				 et->priv->drop_col,
 				 context,
 				 time);
-		if (row >= 0 && col >= 0)
-			gtk_signal_emit (GTK_OBJECT (et),
-					 et_signals [TREE_DRAG_MOTION],
-					 row,
-					 path,
-					 col,
-					 context,
-					 x,
-					 y,
-					 time,
-					 &ret_val);
-	}
-	et->priv->drop_row = row;
-	et->priv->drop_path = path;
-	et->priv->drop_col = col;
-	if (row >= 0 && col >= 0)
 		gtk_signal_emit (GTK_OBJECT (et),
-				 et_signals [TREE_DRAG_DROP],
-				 et->priv->drop_row,
-				 et->priv->drop_path,
-				 et->priv->drop_col,
+				 et_signals [TREE_DRAG_MOTION],
+				 row,
+				 path,
+				 col,
 				 context,
 				 x,
 				 y,
 				 time,
 				 &ret_val);
+	}
+	et->priv->drop_row = row;
+	et->priv->drop_path = path;
+	et->priv->drop_col = col;
+	gtk_signal_emit (GTK_OBJECT (et),
+			 et_signals [TREE_DRAG_DROP],
+			 et->priv->drop_row,
+			 et->priv->drop_path,
+			 et->priv->drop_col,
+			 context,
+			 x,
+			 y,
+			 time,
+			 &ret_val);
 	et->priv->drop_row = -1;
 	et->priv->drop_path = NULL;
 	et->priv->drop_col = -1;
