@@ -67,6 +67,131 @@ e_completion_view_local_key_press_handler (GtkWidget *w, GdkEventKey *ev)
 	return e_completion_view_key_press_handler (w, ev, w);
 }
 
+static void
+e_completion_view_paint (GtkWidget *widget, GdkRectangle *area)
+{
+	gint i;
+
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (E_IS_COMPLETION_VIEW (widget));
+	g_return_if_fail (area != NULL);
+
+	if (!GTK_WIDGET_DRAWABLE (widget))
+		return;
+
+	for (i = 0; i < E_COMPLETION_VIEW (widget)->border_width; ++i) {
+
+		gdk_draw_rectangle (widget->window,
+				    widget->style->black_gc,
+				    FALSE, i, i, 
+				    widget->allocation.width-1-2*i,
+				    widget->allocation.height-1-2*i);
+
+	}
+	
+}
+
+static void
+e_completion_view_draw (GtkWidget *widget, GdkRectangle *area)
+{
+	GtkBin *bin;
+	GdkRectangle child_area;
+
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (E_IS_COMPLETION_VIEW (widget));
+	g_return_if_fail (area != NULL);
+
+	if (GTK_WIDGET_DRAWABLE (widget)) {
+		bin = GTK_BIN (widget);
+
+		e_completion_view_paint (widget, area);
+
+		if (bin->child && gtk_widget_intersect (bin->child, area, &child_area))
+			gtk_widget_draw (bin->child, &child_area);
+	}
+}
+
+static gint
+e_completion_view_expose_event (GtkWidget *widget, GdkEventExpose *event)			  
+{
+	GtkBin *bin;
+	GdkEventExpose child_event;
+
+	g_return_val_if_fail (widget != NULL, FALSE);
+	g_return_val_if_fail (E_IS_COMPLETION_VIEW (widget), FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
+
+	if (GTK_WIDGET_DRAWABLE (widget)) {
+		bin = GTK_BIN (widget);
+
+		e_completion_view_paint (widget, &event->area);
+
+		child_event = *event;
+		if (bin->child &&
+		    GTK_WIDGET_NO_WINDOW (bin->child) &&
+		    gtk_widget_intersect (bin->child, &event->area, &child_event.area))
+			gtk_widget_event (bin->child, (GdkEvent*) &child_event);
+	}
+
+	return FALSE;
+}
+
+static void
+e_completion_view_size_request (GtkWidget *widget, GtkRequisition *requisition)
+{
+	GtkBin *bin;
+
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (E_IS_COMPLETION_VIEW (widget));
+	g_return_if_fail (requisition != NULL);
+
+	bin = GTK_BIN (widget);
+	
+	requisition->width  = 2 * E_COMPLETION_VIEW (widget)->border_width;
+	requisition->height = 2 * E_COMPLETION_VIEW (widget)->border_width;
+
+	if (bin->child && GTK_WIDGET_VISIBLE (bin->child)) {
+		GtkRequisition child_requisition;
+		
+		gtk_widget_size_request (bin->child, &child_requisition);
+		
+		requisition->width += child_requisition.width;
+		requisition->height += child_requisition.height;
+	}
+}
+
+static void
+e_completion_view_size_allocate (GtkWidget *widget, GtkAllocation *allocation)				 
+{
+	GtkBin *bin;
+	GtkAllocation child_allocation;
+
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (E_IS_COMPLETION_VIEW (widget));
+	g_return_if_fail (allocation != NULL);
+
+	bin = GTK_BIN (widget);
+	widget->allocation = *allocation;
+
+	child_allocation.x = E_COMPLETION_VIEW (widget)->border_width;
+	child_allocation.width = MAX(0, (gint)allocation->width - child_allocation.x * 2);
+
+	child_allocation.y = E_COMPLETION_VIEW (widget)->border_width;
+	child_allocation.height = MAX (0, (gint)allocation->height - child_allocation.y * 2);
+
+	if (GTK_WIDGET_REALIZED (widget)) {
+		gdk_window_move_resize (widget->window,
+					allocation->x,
+					allocation->y,
+					allocation->width,
+					allocation->height);
+	}
+	
+	if (bin->child) {
+		gtk_widget_size_allocate (bin->child, &child_allocation);
+	}
+}
+
 GtkType
 e_completion_view_get_type (void)
 {
@@ -143,20 +268,25 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 				GTK_RUN_LAST,
 				object_class->type,
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, activate),
-				gtk_marshal_NONE__POINTER_POINTER,
-				GTK_TYPE_NONE, 2,
-				GTK_TYPE_POINTER, GTK_TYPE_POINTER);
+				gtk_marshal_NONE__POINTER,
+				GTK_TYPE_NONE, 1,
+				GTK_TYPE_POINTER);
 
 	gtk_object_class_add_signals (object_class, e_completion_view_signals, E_COMPLETION_VIEW_LAST_SIGNAL);
 
 	object_class->destroy = e_completion_view_destroy;
 
 	widget_class->key_press_event = e_completion_view_local_key_press_handler;
+	widget_class->draw = e_completion_view_draw;
+	widget_class->expose_event = e_completion_view_expose_event;
+	widget_class->size_request = e_completion_view_size_request;
+	widget_class->size_allocate = e_completion_view_size_allocate;
 }
 
 static void
 e_completion_view_init (ECompletionView *completion)
 {
+	completion->border_width = 2;
 }
 
 static void
@@ -217,8 +347,9 @@ e_completion_view_clear_choices (ECompletionView *cv)
 	g_return_if_fail (cv != NULL);
 	g_return_if_fail (E_IS_COMPLETION_VIEW (cv));
 
-	for (i = cv->choices; i != NULL; i = g_list_next (i))
-		g_free (i->data);
+	for (i = cv->choices; i != NULL; i = g_list_next (i)) {
+		e_completion_match_unref ((ECompletionMatch *) i->data);
+	}
 
 	g_list_free (cv->choices);
 	cv->choices = NULL;
@@ -293,12 +424,11 @@ e_completion_view_set_cursor_row (ECompletionView *cv, gint r)
 static void
 e_completion_view_select (ECompletionView *cv, gint r)
 {
-	const gchar *sel = (const gchar *) g_list_nth_data (cv->choices, r);
-	gpointer extra = e_completion_find_extra_data (cv->completion, sel);
+	ECompletionMatch *match = (ECompletionMatch *) g_list_nth_data (cv->choices, r);
 
 	cv->selection = r;
 	e_completion_view_set_cursor_row (cv, r);
-	gtk_signal_emit (GTK_OBJECT (cv), e_completion_view_signals[E_COMPLETION_VIEW_ACTIVATE], sel, extra);
+	gtk_signal_emit (GTK_OBJECT (cv), e_completion_view_signals[E_COMPLETION_VIEW_ACTIVATE], match);
 }
 
 static gint
@@ -421,13 +551,14 @@ cancel_completion_cb (ECompletion *completion, gpointer user_data)
 }
 
 static void
-completion_cb (ECompletion *completion, const gchar *text, gpointer extra_data, gpointer user_data)
+completion_cb (ECompletion *completion, ECompletionMatch *match, gpointer user_data)
 {
 	ECompletionView *cv = E_COMPLETION_VIEW (user_data);
 	gint r = cv->choice_count;
 	gboolean first = (cv->choices == NULL);
 
-	cv->choices = g_list_append (cv->choices, g_strdup (text));
+	cv->choices = g_list_append (cv->choices, match);
+	e_completion_match_ref (match);
 	++cv->choice_count;
 
 	e_table_model_row_inserted (cv->model, r);
@@ -453,7 +584,7 @@ end_completion_cb (ECompletion *completion, gpointer user_data)
 /*** Table Callbacks ***/
 
 static char *simple_spec = 
-"<ETableSpecification no-headers=\"true\" draw-grid=\"false\" cursor-mode=\"line\"> "
+"<ETableSpecification no-headers=\"true\" draw-grid=\"false\" cursor-mode=\"line\" alternating-row-colors=\"false\">"
 "  <ETableColumn model_col=\"0\" _title=\"Node\" expansion=\"1.0\" "
 "         minimum_width=\"16\" resizable=\"true\" cell=\"string\" "
 "         compare=\"string\"/> "
@@ -486,11 +617,11 @@ static gpointer
 table_value_at (ETableModel *etm, gint c, gint r, gpointer data)
 {
 	ECompletionView *cv = E_COMPLETION_VIEW (data);
-	gpointer p;
+	ECompletionMatch *match;
 
-	p = g_list_nth_data (cv->choices, r);
+	match = (ECompletionMatch *) g_list_nth_data (cv->choices, r);
 
-	return p;
+	return (gpointer) e_completion_match_get_menu_text (match);
 }
 
 static gchar *
@@ -510,8 +641,6 @@ table_click_cb (ETable *et, gint r, gint c, GdkEvent *ev, gpointer data)
 void
 e_completion_view_construct (ECompletionView *cv, ECompletion *completion)
 {
-	GtkWidget *frame;
-
 	g_return_if_fail (cv != NULL);
 	g_return_if_fail (E_IS_COMPLETION_VIEW (cv));
 	g_return_if_fail (completion != NULL);
@@ -557,13 +686,12 @@ e_completion_view_construct (ECompletionView *cv, ECompletion *completion)
 
 	cv->table = e_table_scrolled_new (cv->model, NULL, simple_spec, NULL);
 
+	e_scroll_frame_set_shadow_type (E_SCROLL_FRAME (cv->table), GTK_SHADOW_NONE);
+	e_scroll_frame_set_scrollbar_spacing (E_SCROLL_FRAME (cv->table), 0);
 	e_scroll_frame_set_policy (E_SCROLL_FRAME (cv->table), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 
-	frame = gtk_frame_new (NULL);
-
-	gtk_container_add (GTK_CONTAINER (cv), frame);
-	gtk_container_add (GTK_CONTAINER (frame), cv->table);
-	gtk_widget_show_all (frame);
+	gtk_container_add (GTK_CONTAINER (cv), cv->table);
+	gtk_widget_show_all (cv->table);
 
 	gtk_signal_connect (GTK_OBJECT (e_completion_view_table (cv)),
 			    "click",
@@ -636,7 +764,7 @@ void
 e_completion_view_set_width (ECompletionView *cv, gint width)
 {
 	GtkWidget *w;
-	gint y, r, dummy, line_height;
+	gint y, r, dummy, line_height, final_height;
 	double drop_room, lines;
 
 	g_return_if_fail (cv != NULL);
@@ -676,7 +804,8 @@ e_completion_view_set_width (ECompletionView *cv, gint width)
 	lines = MIN (lines, drop_room);
 
 	/* We reduce the total height by a bit; in practice, this seems to work out well. */
-	gtk_widget_set_usize (w, width, (gint) floor (line_height * lines * 0.97));
+	final_height = (gint) floor (line_height * (0.5 + (float)lines) * 0.97);
+	gtk_widget_set_usize (w, width, final_height);
 }
 
 void
@@ -693,3 +822,4 @@ e_completion_view_set_editable (ECompletionView *cv, gboolean x)
 	e_completion_view_set_cursor_row (cv, -1);
 }
 
+
