@@ -782,7 +782,7 @@ e_entry_enable_completion_full (EEntry *entry, ECompletion *completion, gint del
 
 	gtk_object_ref (GTK_OBJECT (entry->priv->completion_view_popup));
 	gtk_object_sink (GTK_OBJECT (entry->priv->completion_view_popup));
-	gtk_window_set_policy (GTK_WINDOW (entry->priv->completion_view_popup), FALSE, TRUE, FALSE);
+	gtk_window_set_policy (GTK_WINDOW (entry->priv->completion_view_popup), TRUE, TRUE, TRUE);
 	gtk_container_add (GTK_CONTAINER (entry->priv->completion_view_popup), entry->priv->completion_view);
 	gtk_widget_show (entry->priv->completion_view);
 
