@@ -708,7 +708,8 @@ e_shortcut_bar_on_drag_data_get (GtkWidget          *widget,
 {
 	gchar *data;
 
-	if (info == TARGET_SHORTCUT) {
+	if (info == TARGET_SHORTCUT && shortcut_bar->dragged_name
+	    && shortcut_bar->dragged_url) {
 		data = g_strdup_printf ("%s%c%s", shortcut_bar->dragged_name,
 					'\0', shortcut_bar->dragged_url);
 		gtk_selection_data_set (selection_data,	selection_data->target,
